package gateway

import (
	"context"
	"net/http"
	"strings"

	_ "im-microservice/google_api"
	"im-microservice/pb"

	beeLogger "github.com/beego/bee/logger"
	"github.com/grpc-ecosystem/grpc-gateway/runtime"
	"golang.org/x/net/http2"
	"golang.org/x/net/http2/h2c"
	"google.golang.org/grpc"
)

// ProvideHTTP 把gRPC服务转成HTTP服务，让gRPC同时支持HTTP
func ProvideHTTP(endpoint string, grpcServer *grpc.Server) *http.Server {
	ctx := context.Background()
	//获取证书

	//添加证书
	dopts := []grpc.DialOption{grpc.WithInsecure()}
	//新建gwmux，它是grpc-gateway的请求复用器。它将http请求与模式匹配，并调用相应的处理程序。
	gwmux := runtime.NewServeMux()
	//将服务的http处理程序注册到gwmux。处理程序通过endpoint转发请求到grpc端点

	// 好友关系
	err := pb.RegisterUserRelationshipServiceHandlerFromEndpoint(ctx, gwmux, endpoint, dopts)
	if err != nil {
		beeLogger.Log.Fatalf("Register Endpoint err: %v", err)
	}
	// IM配置
	err = pb.RegisterConfigureSeviceHandlerFromEndpoint(ctx, gwmux, endpoint, dopts)
	if err != nil {
		beeLogger.Log.Fatalf("Register Endpoint err: %v", err)
	}

	// 聊天室
	err = pb.RegisterChatRoomServiceHandlerFromEndpoint(ctx, gwmux, endpoint, dopts)
	if err != nil {
		beeLogger.Log.Fatalf("Register Endpoint err: %v", err)
	}

	// 好友申请
	err = pb.RegisterImFriendServiceHandlerFromEndpoint(ctx, gwmux, endpoint, dopts)
	if err != nil {
		beeLogger.Log.Fatalf("Register Endpoint err: %v", err)
	}

	//im用户
	err = pb.RegisterImUserHandlerFromEndpoint(ctx, gwmux, endpoint, dopts)
	if err != nil {
		beeLogger.Log.Fatalf("Register imUser err: %v", err)
	}


	//新建mux，它是http的请求复用器
	mux := http.NewServeMux()
	//注册gwmux
	mux.Handle("/", gwmux)
	beeLogger.Log.Success(endpoint + " HTTP.Listing...")
	return &http.Server{
		Addr:    endpoint,
		Handler: grpcHandlerFunc(grpcServer, mux),
	}
}

var HeaderValues map[string]string

// grpcHandlerFunc 根据不同的请求重定向到指定的Handler处理
func grpcHandlerFunc(grpcServer *grpc.Server, otherHandler http.Handler) http.Handler {
	return h2c.NewHandler(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		if r.Header.Get("Appkey") != "" {
			HeaderValues = map[string]string{
				"Appkey":   r.Header.Get("Appkey"),
				"Nonce":    r.Header.Get("Nonce"),
				"Curtime":  r.Header.Get("Curtime"),
				"Checksum": r.Header.Get("Checksum"),
			}
		}
		ctx := context.WithValue(r.Context(), "Header", HeaderValues)
		if r.ProtoMajor == 2 && strings.Contains(r.Header.Get("Content-Type"), "application/grpc") {
			grpcServer.ServeHTTP(w, r.WithContext(ctx))
		} else {
			otherHandler.ServeHTTP(w, r.WithContext(ctx))
		}

	}), &http2.Server{})
}
