package middleware

import (
	"context"
	"crypto/sha1"
	"encoding/hex"
	"fmt"
	"google.golang.org/grpc/metadata"
	"regexp"
	"strconv"
	"time"

	"im-microservice/helper"

	"google.golang.org/grpc"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

const (
	health_path = "/pb.Health/Check"
)

func permissionError(msg string) error {
	return status.Error(codes.InvalidArgument, msg)
}

func checksum(req map[string]string) error {

	err, AppSecret := helper.GetSecretByKey(req["Appkey"])
	if err != nil {
		return status.Error(codes.NotFound, "Appkey不存在")
	}

	server_time := time.Now().Unix()
	client_time, err := strconv.Atoi(req["Curtime"])
	if err != nil {
		return status.Error(codes.Aborted, err.Error())
	}
	if server_time > int64(client_time) {
		return permissionError("当前客户端时间不能小于服务端时间")
	}

	sha11 := sha1.New()
	sha11.Write([]byte(AppSecret + req["Nonce"] + req["Curtime"]))
	sevice_checksum := hex.EncodeToString(sha11.Sum([]byte(nil)))

	if sevice_checksum != req["Checksum"] {
		return permissionError("加密串不正确")
	}
	return nil
}

func Auth(ctx context.Context, req interface{}, info *grpc.UnaryServerInfo,
	handler grpc.UnaryHandler) (interface{}, error) {

	if info.FullMethod != health_path {
		request_map := make(map[string]string)
		if md, ok := metadata.FromIncomingContext(ctx); ok {
			if _, ok := md["x-forwarded-for"]; ok {
				request_map = ctx.Value("Header").(map[string]string)

				if request_map["Appkey"] == "" {
					return nil, status.Error(codes.NotFound, "Appkey不能为空")
				}
				if request_map["Curtime"] == "" {
					return nil, status.Error(codes.NotFound, "Curtime不能为空")
				}
				if request_map["Nonce"] == "" {
					return nil, status.Error(codes.NotFound, "Nonce不能为空")
				}
				if request_map["Checksum"] == "" {
					return nil, status.Error(codes.NotFound, "Checksum不能为空")
				}
			} else {
				key_r, _ := regexp.Compile(`[^}|^Common:]\s?(.*?):"(.*?)"`)
				result := key_r.FindAllStringSubmatch(fmt.Sprintf("%v", req), -1)
				for _, v := range result {
					request_map[v[1]] = v[2]
				}
			}
			if err := checksum(request_map); err != nil {
				return nil, err
			}
		}
	}
	return handler(ctx, req)
}
