package im_user

import (
	"bytes"
	//"crypto/md5"
	//"encoding/hex"
	"errors"
	"fmt"
	"im-microservice/db"
	"im-microservice/helper"
	"im-microservice/pb"
	"math/rand"
	"time"

	"github.com/astaxie/beego/orm"
	"github.com/segmentio/ksuid"
)

const (
	u_table_name = "im_user"
)

func CreateCaptcha() string {
	return fmt.Sprintf("%04v", rand.New(rand.NewSource(time.Now().UnixNano())).Int31n(10000))
}

func randSeq(n int) string {
	var letters = []rune("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
	b := make([]rune, n)
	for i := range b {
		b[i] = letters[rand.Intn(len(letters))]
	}
	return string(b)
}

func createToken() string {
	var buffer bytes.Buffer
	kid := ksuid.New()
	buffer.WriteString(kid.String())
	buffer.WriteString(randSeq(37))
	return buffer.String()
	// h := md5.New()
	// h.Write([]byte(buffer.String()))
	// return hex.EncodeToString(h.Sum(nil))
}

//通过accid和appkey获取用户信息
func GetImUserByAccidAppkey(accid string, appkey string) (db.ImUser, error) {
	var c_user db.ImUser
	err := db.MysqlClient.QueryTable(u_table_name).Filter("accid", accid).Filter("appkey", appkey).One(&c_user)
	if err != nil {
		return c_user, err
	}
	return c_user, nil
}

//im用户注册
func CreateImUser(in *pb.ImUserRequest) (db.ImUser, error) {
	token := createToken()
	var c_user db.ImUser
	c_user.Accid = in.GetAccid()
	c_user.Appkey = in.Appkey
	c_user.Name = in.GetName()
	c_user.Icon = in.GetIcon()
	c_user.Token = token
	c_user.Sign = in.GetSign()
	c_user.Email = in.GetEmail()
	c_user.Birth = in.GetBirth()
	c_user.Mobile = in.GetMobile()
	c_user.Gender = in.GetGender()
	c_user.Ex = in.GetEx()
	c_user.Createtime = helper.GetNowTime()
	_, err := db.MysqlClient.Insert(&c_user)
	if err != nil {
		return c_user, err
	}

	return c_user, nil
}

//修改用户信息
func UpdateImUser(accid string, appkey string, m map[string]interface{}) (int64, error) {
	orm_params := make(orm.Params)
	for k, v := range m {
		switch v.(type) {
		case int:
			i_v := v.(int)
			orm_params[k] = i_v
		case string:
			s_v := v.(string)
			orm_params[k] = s_v
		case int64:
			c_v := v.(int64)
			orm_params[k] = c_v
		}
	}
	res, err := db.MysqlClient.QueryTable(u_table_name).Filter("accid", accid).Filter("appkey", appkey).Update(orm_params)
	if err != nil {
		return 0, err
	}
	return res, nil
}

//获取用户列表
func GetImUserList(request *pb.ImUserListRequest) (results map[string]interface{}, err error) {
	var (
		mobile           = request.GetMobile()
		name             = request.GetName()
		valid            = request.GetValid()
		mute_estoppel    = request.GetMuteEstoppel()
		mute_audio_video = request.GetMuteAudioVideo()
		page             = request.GetPage()
		edition          = request.GetEdition()
	)
	if page == 0 {
		page = 1
	}
	count_db := db.MysqlClient.QueryTable(u_table_name)
	if name != "" {
		count_db = count_db.Filter("name__icontains", name)
	}
	if mobile != "" {
		count_db = count_db.Filter("mobile", mobile)
	}
	if valid != 0 {
		count_db = count_db.Filter("valid", valid)
	}
	if mute_estoppel != 0 {
		count_db = count_db.Filter("mute_estoppel", mute_estoppel)
	}
	if mute_audio_video != 0 {
		count_db = count_db.Filter("mute_audio_video", mute_audio_video)
	}

	if edition != "" {
		count_db = count_db.Filter("edition", edition)
	}
	ctn, err := count_db.Count()
	if err != nil {
		return
	}
	if ctn == 0 {
		err = errors.New("没有查询到任何数据")
		return
	}
	//获取
	offset, total_page := helper.Paginate(ctn, 20, page)

	// 查询数据
	var im_user []db.ImUser
	if _, err = count_db.Limit(20, offset).All(&im_user); err != nil {
		return
	}
	results = make(map[string]interface{})
	results["list"] = im_user
	results["total_page"] = total_page
	results["total_count"] = ctn
	results["page"] = page
	results["page_size"] = 20
	return
}
