package im_user_relationship

import (
	"context"
	"fmt"

	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"

	"im-microservice/helper"
	"im-microservice/pb"
	"im-microservice/sevice/im_user"
)

type UserRelationshipService struct {
	pb.UnimplementedUserRelationshipServiceServer
}

func (us *UserRelationshipService) Add(ctx context.Context,
	in *pb.UserRelationshipAddRequest) (reply *pb.UserRelationshipAddReply, err error) {
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	//添加好友
	if err = AddUserRelationship(in); err != nil {
		return
	}
	// 更改用户版本信息
	u_map := make(map[string]interface{})
	u_map["edition"] = helper.GetNowTime()
	_, _ = im_user.UpdateImUser(in.Accid, in.Appkey, u_map)

	// 更改请求用户版本信息
	_, _ = im_user.UpdateImUser(in.Faccid, in.Appkey, u_map)

	reply = &pb.UserRelationshipAddReply{}
	return
}

func (us *UserRelationshipService) Delete(ctx context.Context,
	in *pb.UserRelationshipDelRequest) (reply *pb.UserRelationshipAddReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	// 删除好友
	if err = DBDelete(in); err != nil {
		return
	}
	// 更改用户版本信息
	u_map := make(map[string]interface{})
	u_map["edition"] = helper.GetNowTime()
	_, _ = im_user.UpdateImUser(in.Accid, in.Appkey, u_map)

	// 更改被添加用户版本信息
	_, _ = im_user.UpdateImUser(in.Faccid, in.Appkey, u_map)

	reply = &pb.UserRelationshipAddReply{}
	return
}

func (us *UserRelationshipService) All(ctx context.Context,
	in *pb.UserRelationshipListRequest) (reply *pb.UserRelationshipListReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	results, err := DBAll(in)
	if err != nil {
		err = status.Error(codes.Internal, err.Error())
		return
	}
	reply = &pb.UserRelationshipListReply{}
	for _, v := range results {
		reply.List = append(reply.List, &pb.UserRelationshipInfo{
			Faccid:     v.Faccid,
			Updatetime: v.Updatetime,
		})
	}
	return
}

func (us *UserRelationshipService) SetSpecialRelation(ctx context.Context,
	in *pb.SetSpecialRelationReq) (reply *pb.UserRelationshipAddReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	if err = SetSpecialRelationDb(in); err != nil {
		err = status.Error(codes.Internal, err.Error())
		return
	}
	reply = &pb.UserRelationshipAddReply{}
	return
}

func (us *UserRelationshipService) ListBlackAndMuteList(ctx context.Context,
	in *pb.ListBlackAndMuteListReq) (reply *pb.ListBlackAndMuteListReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}

	blacklist, mutelist, err := ListBlackAndMuteListDB(in)
	if err != nil {
		err = status.Error(codes.Internal, err.Error())
		return
	}
	reply = &pb.ListBlackAndMuteListReply{}

	if len(blacklist) > 0 {
		for _, v := range blacklist {
			reply.Blacklist = append(reply.Blacklist,
				&pb.UserRelationshipInfo{Updatetime: v.Updatetime, Faccid: v.Faccid})
		}
	}
	fmt.Println("123123")

	if len(mutelist) > 0 {
		for _, v := range mutelist {
			reply.Mutelist = append(reply.Mutelist,
				&pb.UserRelationshipInfo{Updatetime: v.Updatetime, Faccid: v.Faccid})
		}
	}
	return
}


func (us *UserRelationshipService) One (ctx context.Context,
	in *pb.FriendRelationshipReq) ( reply *pb.FriendRelationshipReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}

	result, err := GetUserRelationshipOne(in.Accid, in.Faccid, in.Appkey)
	if err != nil {
		err = status.Error(codes.NotFound, err.Error())
		return
	}
	reply = &pb.FriendRelationshipReply{
		Type:       int64(result.Type),
		Msg:        result.Msg,
		Blacklist:  int64(result.Blacklist),
		Mute:       int64(result.Mute),
		Serverex:   result.Serverex,
		Createtime: result.Createtime,
		Updatetime: result.Updatetime,
	}
	return
}